@@ -1,5 +1,3 @@
-extern crate zip;
-
 use std::fs;
 use std::io::BufReader;
 
@@ -21,6 +19,7 @@ fn real_main() -> i32 {
 
     for i in 0..archive.len() {
         let file = archive.by_index(i).unwrap();
+        #[allow(deprecated)]
         let outpath = file.sanitized_name();
 
         {
@@ -31,9 +30,18 @@ fn real_main() -> i32 {
         }
 
         if (&*file.name()).ends_with('/') {
-            println!("Entry {} is a directory with name \"{}\"", i, outpath.as_path().display());
+            println!(
+                "Entry {} is a directory with name \"{}\"",
+                i,
+                outpath.as_path().display()
+            );
         } else {
-            println!("Entry {} is a file with name \"{}\" ({} bytes)", i, outpath.as_path().display(), file.size());
+            println!(
+                "Entry {} is a file with name \"{}\" ({} bytes)",
+                i,
+                outpath.as_path().display(),
+                file.size()
+            );
         }
     }
     return 0;
