@@ -1,73 +1,77 @@
-//! Structs for creating a new zip archive
+//! Types for creating ZIP archives
 
-use compression::CompressionMethod;
-use types::{ZipFileData, System, DEFAULT_VERSION, DateTime};
-use spec;
+use crate::compression::CompressionMethod;
+use crate::result::{ZipError, ZipResult};
+use crate::spec;
+use crate::types::{DateTime, System, ZipFileData, DEFAULT_VERSION};
+use byteorder::{LittleEndian, WriteBytesExt};
 use crc32fast::Hasher;
-use result::{ZipResult, ZipError};
 use std::default::Default;
 use std::io;
 use std::io::prelude::*;
 use std::mem;
-#[cfg(feature = "time")]
-use time;
-use podio::{WritePodExt, LittleEndian};
 
-#[cfg(feature = "deflate")]
-use flate2;
-#[cfg(feature = "deflate")]
+#[cfg(any(
+    feature = "deflate",
+    feature = "deflate-miniz",
+    feature = "deflate-zlib"
+))]
 use flate2::write::DeflateEncoder;
 
-#[cfg(feature = "bzip2")]
-use bzip2;
 #[cfg(feature = "bzip2")]
 use bzip2::write::BzEncoder;
 
-enum GenericZipWriter<W: Write + io::Seek>
-{
+enum GenericZipWriter<W: Write + io::Seek> {
     Closed,
     Storer(W),
-    #[cfg(feature = "deflate")]
+    #[cfg(any(
+        feature = "deflate",
+        feature = "deflate-miniz",
+        feature = "deflate-zlib"
+    ))]
     Deflater(DeflateEncoder<W>),
     #[cfg(feature = "bzip2")]
     Bzip2(BzEncoder<W>),
 }
 
-/// Generator for ZIP files.
+/// ZIP archive generator
+///
+/// Handles the bookkeeping involved in building an archive, and provides an
+/// API to edit its contents.
 ///
 /// ```
-/// fn doit() -> zip::result::ZipResult<()>
-/// {
-///     use std::io::Write;
+/// # fn doit() -> zip::result::ZipResult<()>
+/// # {
+/// # use zip::ZipWriter;
+/// use std::io::Write;
+/// use zip::write::FileOptions;
 ///
-///     // For this example we write to a buffer, but normally you should use a File
-///     let mut buf: &mut [u8] = &mut [0u8; 65536];
-///     let mut w = std::io::Cursor::new(buf);
-///     let mut zip = zip::ZipWriter::new(w);
+/// // We use a buffer here, though you'd normally use a `File`
+/// let mut buf = [0; 65536];
+/// let mut zip = zip::ZipWriter::new(std::io::Cursor::new(&mut buf[..]));
 ///
-///     let options = zip::write::FileOptions::default().compression_method(zip::CompressionMethod::Stored);
-///     zip.start_file("hello_world.txt", options)?;
-///     zip.write(b"Hello, World!")?;
+/// let options = zip::write::FileOptions::default().compression_method(zip::CompressionMethod::Stored);
+/// zip.start_file("hello_world.txt", options)?;
+/// zip.write(b"Hello, World!")?;
 ///
-///     // Optionally finish the zip. (this is also done on drop)
-///     zip.finish()?;
+/// // Apply the changes you've made.
+/// // Dropping the `ZipWriter` will have the same effect, but may silently fail
+/// zip.finish()?;
 ///
-///     Ok(())
-/// }
-///
-/// println!("Result: {:?}", doit().unwrap());
+/// # Ok(())
+/// # }
+/// # doit().unwrap();
 /// ```
-pub struct ZipWriter<W: Write + io::Seek>
-{
+pub struct ZipWriter<W: Write + io::Seek> {
     inner: GenericZipWriter<W>,
     files: Vec<ZipFileData>,
     stats: ZipWriterStats,
     writing_to_file: bool,
+    comment: String,
 }
 
 #[derive(Default)]
-struct ZipWriterStats
-{
+struct ZipWriterStats {
     hasher: Hasher,
     start: u64,
     bytes_written: u64,
@@ -85,10 +89,22 @@ impl FileOptions {
     /// Construct a new FileOptions object
     pub fn default() -> FileOptions {
         FileOptions {
-            #[cfg(feature = "deflate")]      compression_method: CompressionMethod::Deflated,
-            #[cfg(not(feature = "deflate"))] compression_method: CompressionMethod::Stored,
-            #[cfg(feature = "time")]      last_modified_time: DateTime::from_time(time::now()).unwrap_or(DateTime::default()),
-            #[cfg(not(feature = "time"))] last_modified_time: DateTime::default(),
+            #[cfg(any(
+                feature = "deflate",
+                feature = "deflate-miniz",
+                feature = "deflate-zlib"
+            ))]
+            compression_method: CompressionMethod::Deflated,
+            #[cfg(not(any(
+                feature = "deflate",
+                feature = "deflate-miniz",
+                feature = "deflate-zlib"
+            )))]
+            compression_method: CompressionMethod::Stored,
+            #[cfg(feature = "time")]
+            last_modified_time: DateTime::from_time(time::now()).unwrap_or_default(),
+            #[cfg(not(feature = "time"))]
+            last_modified_time: DateTime::default(),
             permissions: None,
         }
     }
@@ -129,63 +145,73 @@ impl Default for FileOptions {
     }
 }
 
-impl<W: Write+io::Seek> Write for ZipWriter<W>
-{
-    fn write(&mut self, buf: &[u8]) -> io::Result<usize>
-    {
-        if !self.writing_to_file { return Err(io::Error::new(io::ErrorKind::Other, "No file has been started")) }
-        match self.inner.ref_mut()
-        {
+impl<W: Write + io::Seek> Write for ZipWriter<W> {
+    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
+        if !self.writing_to_file {
+            return Err(io::Error::new(
+                io::ErrorKind::Other,
+                "No file has been started",
+            ));
+        }
+        match self.inner.ref_mut() {
             Some(ref mut w) => {
                 let write_result = w.write(buf);
                 if let Ok(count) = write_result {
                     self.stats.update(&buf[0..count]);
                 }
                 write_result
-
             }
-            None => Err(io::Error::new(io::ErrorKind::BrokenPipe, "ZipWriter was already closed")),
+            None => Err(io::Error::new(
+                io::ErrorKind::BrokenPipe,
+                "ZipWriter was already closed",
+            )),
         }
     }
 
-    fn flush(&mut self) -> io::Result<()>
-    {
-        match self.inner.ref_mut()
-        {
+    fn flush(&mut self) -> io::Result<()> {
+        match self.inner.ref_mut() {
             Some(ref mut w) => w.flush(),
-            None => Err(io::Error::new(io::ErrorKind::BrokenPipe, "ZipWriter was already closed")),
+            None => Err(io::Error::new(
+                io::ErrorKind::BrokenPipe,
+                "ZipWriter was already closed",
+            )),
         }
     }
 }
 
-impl ZipWriterStats
-{
-    fn update(&mut self, buf: &[u8])
-    {
+impl ZipWriterStats {
+    fn update(&mut self, buf: &[u8]) {
         self.hasher.update(buf);
         self.bytes_written += buf.len() as u64;
     }
 }
 
-impl<W: Write+io::Seek> ZipWriter<W>
-{
-    /// Initializes the ZipWriter.
+impl<W: Write + io::Seek> ZipWriter<W> {
+    /// Initializes the archive.
     ///
-    /// Before writing to this object, the start_file command should be called.
-    pub fn new(inner: W) -> ZipWriter<W>
-    {
-        ZipWriter
-        {
+    /// Before writing to this object, the [`ZipWriter::start_file`] function should be called.
+    pub fn new(inner: W) -> ZipWriter<W> {
+        ZipWriter {
             inner: GenericZipWriter::Storer(inner),
             files: Vec::new(),
             stats: Default::default(),
             writing_to_file: false,
+            comment: String::new(),
         }
     }
 
+    /// Set ZIP archive comment.
+    pub fn set_comment<S>(&mut self, comment: S)
+    where
+        S: Into<String>,
+    {
+        self.comment = comment.into();
+    }
+
     /// Start a new file for with the requested options.
     fn start_entry<S>(&mut self, name: S, options: FileOptions) -> ZipResult<()>
-        where S: Into<String>
+    where
+        S: Into<String>,
     {
         self.finish_file()?;
 
@@ -196,8 +222,7 @@ impl<W: Write+io::Seek> ZipWriter<W>
             let permissions = options.permissions.unwrap_or(0o100644);
             let file_name = name.into();
             let file_name_raw = file_name.clone().into_bytes();
-            let mut file = ZipFileData
-            {
+            let mut file = ZipFileData {
                 system: System::Unix,
                 version_made_by: DEFAULT_VERSION,
                 encrypted: false,
@@ -206,11 +231,12 @@ impl<W: Write+io::Seek> ZipWriter<W>
                 crc32: 0,
                 compressed_size: 0,
                 uncompressed_size: 0,
-                file_name: file_name,
-                file_name_raw: file_name_raw,
+                file_name,
+                file_name_raw,
                 file_comment: String::new(),
-                header_start: header_start,
+                header_start,
                 data_start: 0,
+                central_header_start: 0,
                 external_attributes: permissions << 16,
             };
             write_local_file_header(writer, &file)?;
@@ -230,13 +256,11 @@ impl<W: Write+io::Seek> ZipWriter<W>
         Ok(())
     }
 
-    fn finish_file(&mut self) -> ZipResult<()>
-    {
+    fn finish_file(&mut self) -> ZipResult<()> {
         self.inner.switch_to(CompressionMethod::Stored)?;
         let writer = self.inner.get_plain();
 
-        let file = match self.files.last_mut()
-        {
+        let file = match self.files.last_mut() {
             None => return Ok(()),
             Some(f) => f,
         };
@@ -253,9 +277,12 @@ impl<W: Write+io::Seek> ZipWriter<W>
         Ok(())
     }
 
-    /// Starts a file.
+    /// Create a file in the archive and start writing its' contents.
+    ///
+    /// The data should be written using the [`io::Write`] implementation on this [`ZipWriter`]
     pub fn start_file<S>(&mut self, name: S, mut options: FileOptions) -> ZipResult<()>
-        where S: Into<String>
+    where
+        S: Into<String>,
     {
         if options.permissions.is_none() {
             options.permissions = Some(0o644);
@@ -270,7 +297,15 @@ impl<W: Write+io::Seek> ZipWriter<W>
     ///
     /// This function ensures that the '/' path seperator is used. It also ignores all non 'Normal'
     /// Components, such as a starting '/' or '..' and '.'.
-    pub fn start_file_from_path(&mut self, path: &std::path::Path, options: FileOptions) -> ZipResult<()> {
+    #[deprecated(
+        since = "0.5.7",
+        note = "by stripping `..`s from the path, the meaning of paths can change. Use `start_file` instead."
+    )]
+    pub fn start_file_from_path(
+        &mut self,
+        path: &std::path::Path,
+        options: FileOptions,
+    ) -> ZipResult<()> {
         self.start_file(path_to_string(path), options)
     }
 
@@ -278,7 +313,8 @@ impl<W: Write+io::Seek> ZipWriter<W>
     ///
     /// You can't write data to the file afterwards.
     pub fn add_directory<S>(&mut self, name: S, mut options: FileOptions) -> ZipResult<()>
-        where S: Into<String>
+    where
+        S: Into<String>,
     {
         if options.permissions.is_none() {
             options.permissions = Some(0o755);
@@ -302,44 +338,48 @@ impl<W: Write+io::Seek> ZipWriter<W>
     ///
     /// This function ensures that the '/' path seperator is used. It also ignores all non 'Normal'
     /// Components, such as a starting '/' or '..' and '.'.
-    pub fn add_directory_from_path(&mut self, path: &std::path::Path, options: FileOptions) -> ZipResult<()> {
-        self.add_directory(path_to_string(path.into()), options)
+    #[deprecated(
+        since = "0.5.7",
+        note = "by stripping `..`s from the path, the meaning of paths can change. Use `add_directory` instead."
+    )]
+    pub fn add_directory_from_path(
+        &mut self,
+        path: &std::path::Path,
+        options: FileOptions,
+    ) -> ZipResult<()> {
+        self.add_directory(path_to_string(path), options)
     }
 
     /// Finish the last file and write all other zip-structures
     ///
     /// This will return the writer, but one should normally not append any data to the end of the file.
     /// Note that the zipfile will also be finished on drop.
-    pub fn finish(&mut self) -> ZipResult<W>
-    {
+    pub fn finish(&mut self) -> ZipResult<W> {
         self.finalize()?;
         let inner = mem::replace(&mut self.inner, GenericZipWriter::Closed);
         Ok(inner.unwrap())
     }
 
-    fn finalize(&mut self) -> ZipResult<()>
-    {
+    fn finalize(&mut self) -> ZipResult<()> {
         self.finish_file()?;
 
         {
             let writer = self.inner.get_plain();
 
             let central_start = writer.seek(io::SeekFrom::Current(0))?;
-            for file in self.files.iter()
-            {
+            for file in self.files.iter() {
                 write_central_directory_header(writer, file)?;
             }
             let central_size = writer.seek(io::SeekFrom::Current(0))? - central_start;
 
-            let footer = spec::CentralDirectoryEnd
-            {
+            let footer = spec::CentralDirectoryEnd {
                 disk_number: 0,
                 disk_with_central_directory: 0,
                 number_of_files_on_this_disk: self.files.len() as u16,
                 number_of_files: self.files.len() as u16,
                 central_directory_size: central_size as u32,
                 central_directory_offset: central_start as u32,
-                zip_file_comment: b"zip-rs".to_vec(),
+                zip_file_comment: self.comment.as_bytes().to_vec(),
             };
 
             footer.write(writer)?;
@@ -349,12 +389,9 @@ impl<W: Write+io::Seek> ZipWriter<W>
     }
 }
 
-impl<W: Write+io::Seek> Drop for ZipWriter<W>
-{
-    fn drop(&mut self)
-    {
-        if !self.inner.is_closed()
-        {
+impl<W: Write + io::Seek> Drop for ZipWriter<W> {
+    fn drop(&mut self) {
+        if !self.inner.is_closed() {
             if let Err(e) = self.finalize() {
                 let _ = write!(&mut io::stderr(), "ZipWriter drop failed: {:?}", e);
             }
@@ -362,63 +399,89 @@ impl<W: Write+io::Seek> Drop for ZipWriter<W>
     }
 }
 
-impl<W: Write+io::Seek> GenericZipWriter<W>
-{
-    fn switch_to(&mut self, compression: CompressionMethod) -> ZipResult<()>
-    {
+impl<W: Write + io::Seek> GenericZipWriter<W> {
+    fn switch_to(&mut self, compression: CompressionMethod) -> ZipResult<()> {
         match self.current_compression() {
             Some(method) if method == compression => return Ok(()),
-            None => Err(io::Error::new(io::ErrorKind::BrokenPipe, "ZipWriter was already closed"))?,
-            _ => {},
+            None => {
+                return Err(io::Error::new(
+                    io::ErrorKind::BrokenPipe,
+                    "ZipWriter was already closed",
+                )
+                .into())
+            }
+            _ => {}
         }
 
-        let bare = match mem::replace(self, GenericZipWriter::Closed)
-        {
+        let bare = match mem::replace(self, GenericZipWriter::Closed) {
             GenericZipWriter::Storer(w) => w,
-            #[cfg(feature = "deflate")]
+            #[cfg(any(
+                feature = "deflate",
+                feature = "deflate-miniz",
+                feature = "deflate-zlib"
+            ))]
             GenericZipWriter::Deflater(w) => w.finish()?,
             #[cfg(feature = "bzip2")]
             GenericZipWriter::Bzip2(w) => w.finish()?,
-            GenericZipWriter::Closed => Err(io::Error::new(io::ErrorKind::BrokenPipe, "ZipWriter was already closed"))?,
+            GenericZipWriter::Closed => {
+                return Err(io::Error::new(
+                    io::ErrorKind::BrokenPipe,
+                    "ZipWriter was already closed",
+                )
+                .into())
+            }
         };
 
-        *self = match compression
-        {
-            CompressionMethod::Stored => GenericZipWriter::Storer(bare),
-            #[cfg(feature = "deflate")]
-            CompressionMethod::Deflated => GenericZipWriter::Deflater(DeflateEncoder::new(bare, flate2::Compression::default())),
-            #[cfg(feature = "bzip2")]
-            CompressionMethod::Bzip2 => GenericZipWriter::Bzip2(BzEncoder::new(bare, bzip2::Compression::Default)),
-            CompressionMethod::Unsupported(..) => return Err(ZipError::UnsupportedArchive("Unsupported compression")),
+        *self = {
+            #[allow(deprecated)]
+            match compression {
+                CompressionMethod::Stored => GenericZipWriter::Storer(bare),
+                #[cfg(any(
+                    feature = "deflate",
+                    feature = "deflate-miniz",
+                    feature = "deflate-zlib"
+                ))]
+                CompressionMethod::Deflated => GenericZipWriter::Deflater(DeflateEncoder::new(
+                    bare,
+                    flate2::Compression::default(),
+                )),
+                #[cfg(feature = "bzip2")]
+                CompressionMethod::Bzip2 => {
+                    GenericZipWriter::Bzip2(BzEncoder::new(bare, bzip2::Compression::Default))
+                }
+                CompressionMethod::Unsupported(..) => {
+                    return Err(ZipError::UnsupportedArchive("Unsupported compression"))
+                }
+            }
         };
 
         Ok(())
     }
 
-    fn ref_mut(&mut self) -> Option<&mut Write> {
+    fn ref_mut(&mut self) -> Option<&mut dyn Write> {
         match *self {
-            GenericZipWriter::Storer(ref mut w) => Some(w as &mut Write),
-            #[cfg(feature = "deflate")]
-            GenericZipWriter::Deflater(ref mut w) => Some(w as &mut Write),
+            GenericZipWriter::Storer(ref mut w) => Some(w as &mut dyn Write),
+            #[cfg(any(
+                feature = "deflate",
+                feature = "deflate-miniz",
+                feature = "deflate-zlib"
+            ))]
+            GenericZipWriter::Deflater(ref mut w) => Some(w as &mut dyn Write),
             #[cfg(feature = "bzip2")]
-            GenericZipWriter::Bzip2(ref mut w) => Some(w as &mut Write),
+            GenericZipWriter::Bzip2(ref mut w) => Some(w as &mut dyn Write),
             GenericZipWriter::Closed => None,
         }
     }
 
-    fn is_closed(&self) -> bool
-    {
-        match *self
-        {
+    fn is_closed(&self) -> bool {
+        match *self {
             GenericZipWriter::Closed => true,
             _ => false,
         }
     }
 
-    fn get_plain(&mut self) -> &mut W
-    {
-        match *self
-        {
+    fn get_plain(&mut self) -> &mut W {
+        match *self {
             GenericZipWriter::Storer(ref mut w) => w,
             _ => panic!("Should have switched to stored beforehand"),
         }
@@ -427,7 +490,11 @@ impl<W: Write+io::Seek> GenericZipWriter<W>
     fn current_compression(&self) -> Option<CompressionMethod> {
         match *self {
             GenericZipWriter::Storer(..) => Some(CompressionMethod::Stored),
-            #[cfg(feature = "deflate")]
+            #[cfg(any(
+                feature = "deflate",
+                feature = "deflate-miniz",
+                feature = "deflate-zlib"
+            ))]
             GenericZipWriter::Deflater(..) => Some(CompressionMethod::Deflated),
             #[cfg(feature = "bzip2")]
             GenericZipWriter::Bzip2(..) => Some(CompressionMethod::Bzip2),
@@ -435,26 +502,28 @@ impl<W: Write+io::Seek> GenericZipWriter<W>
         }
     }
 
-    fn unwrap(self) -> W
-    {
-        match self
-        {
+    fn unwrap(self) -> W {
+        match self {
             GenericZipWriter::Storer(w) => w,
             _ => panic!("Should have switched to stored beforehand"),
         }
     }
 }
 
-fn write_local_file_header<T: Write>(writer: &mut T, file: &ZipFileData) -> ZipResult<()>
-{
+fn write_local_file_header<T: Write>(writer: &mut T, file: &ZipFileData) -> ZipResult<()> {
     // local file header signature
     writer.write_u32::<LittleEndian>(spec::LOCAL_FILE_HEADER_SIGNATURE)?;
     // version needed to extract
     writer.write_u16::<LittleEndian>(file.version_needed())?;
     // general purpose bit flag
-    let flag = if !file.file_name.is_ascii() { 1u16 << 11 } else { 0 };
+    let flag = if !file.file_name.is_ascii() {
+        1u16 << 11
+    } else {
+        0
+    };
     writer.write_u16::<LittleEndian>(flag)?;
     // Compression method
+    #[allow(deprecated)]
     writer.write_u16::<LittleEndian>(file.compression_method.to_u16())?;
     // last mod file time and last mod file date
     writer.write_u16::<LittleEndian>(file.last_modified_time.timepart())?;
@@ -478,9 +547,11 @@ fn write_local_file_header<T: Write>(writer: &mut T, file: &ZipFileData) -> ZipR
     Ok(())
 }
 
-fn update_local_file_header<T: Write+io::Seek>(writer: &mut T, file: &ZipFileData) -> ZipResult<()>
-{
-    const CRC32_OFFSET : u64 = 14;
+fn update_local_file_header<T: Write + io::Seek>(
+    writer: &mut T,
+    file: &ZipFileData,
+) -> ZipResult<()> {
+    const CRC32_OFFSET: u64 = 14;
     writer.seek(io::SeekFrom::Start(file.header_start + CRC32_OFFSET))?;
     writer.write_u32::<LittleEndian>(file.crc32)?;
     writer.write_u32::<LittleEndian>(file.compressed_size as u32)?;
@@ -488,8 +559,7 @@ fn update_local_file_header<T: Write+io::Seek>(writer: &mut T, file: &ZipFileDat
     Ok(())
 }
 
-fn write_central_directory_header<T: Write>(writer: &mut T, file: &ZipFileData) -> ZipResult<()>
-{
+fn write_central_directory_header<T: Write>(writer: &mut T, file: &ZipFileData) -> ZipResult<()> {
     // central file header signature
     writer.write_u32::<LittleEndian>(spec::CENTRAL_DIRECTORY_HEADER_SIGNATURE)?;
     // version made by
@@ -498,9 +568,14 @@ fn write_central_directory_header<T: Write>(writer: &mut T, file: &ZipFileData)
     // version needed to extract
     writer.write_u16::<LittleEndian>(file.version_needed())?;
     // general puprose bit flag
-    let flag = if !file.file_name.is_ascii() { 1u16 << 11 } else { 0 };
+    let flag = if !file.file_name.is_ascii() {
+        1u16 << 11
+    } else {
+        0
+    };
     writer.write_u16::<LittleEndian>(flag)?;
     // compression method
+    #[allow(deprecated)]
     writer.write_u16::<LittleEndian>(file.compression_method.to_u16())?;
     // last mod file time + date
     writer.write_u16::<LittleEndian>(file.last_modified_time.timepart())?;
@@ -536,8 +611,7 @@ fn write_central_directory_header<T: Write>(writer: &mut T, file: &ZipFileData)
     Ok(())
 }
 
-fn build_extra_field(_file: &ZipFileData) -> ZipResult<Vec<u8>>
-{
+fn build_extra_field(_file: &ZipFileData) -> ZipResult<Vec<u8>> {
     let writer = Vec::new();
     // Future work
     Ok(writer)
@@ -546,14 +620,11 @@ fn build_extra_field(_file: &ZipFileData) -> ZipResult<Vec<u8>>
 fn path_to_string(path: &std::path::Path) -> String {
     let mut path_str = String::new();
     for component in path.components() {
-        match component {
-            std::path::Component::Normal(os_str) => {
-                if path_str.len() != 0 {
-                    path_str.push('/');
-                }
-                path_str.push_str(&*os_str.to_string_lossy());
+        if let std::path::Component::Normal(os_str) = component {
+            if !path_str.is_empty() {
+                path_str.push('/');
             }
-            _ => (),
+            path_str.push_str(&*os_str.to_string_lossy());
         }
     }
     path_str
@@ -561,35 +632,50 @@ fn path_to_string(path: &std::path::Path) -> String {
 
 #[cfg(test)]
 mod test {
+    use super::{FileOptions, ZipWriter};
+    use crate::compression::CompressionMethod;
+    use crate::types::DateTime;
     use std::io;
     use std::io::Write;
-    use types::DateTime;
-    use super::{FileOptions, ZipWriter};
-    use compression::CompressionMethod;
 
     #[test]
     fn write_empty_zip() {
         let mut writer = ZipWriter::new(io::Cursor::new(Vec::new()));
+        writer.set_comment("ZIP");
         let result = writer.finish().unwrap();
-        assert_eq!(result.get_ref().len(), 28);
-        assert_eq!(*result.get_ref(), [80, 75, 5, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 122, 105, 112, 45, 114, 115]);
+        assert_eq!(result.get_ref().len(), 25);
+        assert_eq!(
+            *result.get_ref(),
+            [80, 75, 5, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 90, 73, 80]
+        );
     }
 
     #[test]
     fn write_zip_dir() {
         let mut writer = ZipWriter::new(io::Cursor::new(Vec::new()));
-        writer.add_directory("test", FileOptions::default().last_modified_time(
-            DateTime::from_date_and_time(2018, 8, 15, 20, 45, 6).unwrap()
-        )).unwrap();
-        assert!(writer.write(b"writing to a directory is not allowed, and will not write any data").is_err());
+        writer
+            .add_directory(
+                "test",
+                FileOptions::default().last_modified_time(
+                    DateTime::from_date_and_time(2018, 8, 15, 20, 45, 6).unwrap(),
+                ),
+            )
+            .unwrap();
+        assert!(writer
+            .write(b"writing to a directory is not allowed, and will not write any data")
+            .is_err());
         let result = writer.finish().unwrap();
-        assert_eq!(result.get_ref().len(), 114);
-        assert_eq!(*result.get_ref(), &[
-            80u8, 75, 3, 4, 20, 0, 0, 0, 0, 0, 163, 165, 15, 77, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 116,
-            101, 115, 116, 47, 80, 75, 1, 2, 46, 3, 20, 0, 0, 0, 0, 0, 163, 165, 15, 77, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-            0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 237, 65, 0, 0, 0, 0, 116, 101, 115, 116, 47, 80, 75, 5, 6,
-            0, 0, 0, 0, 1, 0, 1, 0, 51, 0, 0, 0, 35, 0, 0, 0, 6, 0, 122, 105, 112, 45, 114, 115
-        ] as &[u8]);
+        assert_eq!(result.get_ref().len(), 108);
+        assert_eq!(
+            *result.get_ref(),
+            &[
+                80u8, 75, 3, 4, 20, 0, 0, 0, 0, 0, 163, 165, 15, 77, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                0, 0, 5, 0, 0, 0, 116, 101, 115, 116, 47, 80, 75, 1, 2, 46, 3, 20, 0, 0, 0, 0, 0,
+                163, 165, 15, 77, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                0, 0, 237, 65, 0, 0, 0, 0, 116, 101, 115, 116, 47, 80, 75, 5, 6, 0, 0, 0, 0, 1, 0,
+                1, 0, 51, 0, 0, 0, 35, 0, 0, 0, 0, 0,
+            ] as &[u8]
+        );
     }
 
     #[test]
@@ -601,9 +687,12 @@ mod test {
             permissions: Some(33188),
         };
         writer.start_file("mimetype", options).unwrap();
-        writer.write(b"application/vnd.oasis.opendocument.text").unwrap();
+        writer
+            .write(b"application/vnd.oasis.opendocument.text")
+            .unwrap();
         let result = writer.finish().unwrap();
-        assert_eq!(result.get_ref().len(), 159);
+
+        assert_eq!(result.get_ref().len(), 153);
         let mut v = Vec::new();
         v.extend_from_slice(include_bytes!("../tests/data/mimetype.zip"));
         assert_eq!(result.get_ref(), &v);
@@ -612,8 +701,10 @@ mod test {
     #[test]
     fn path_to_string() {
         let mut path = std::path::PathBuf::new();
-        #[cfg(windows)] path.push(r"C:\");
-        #[cfg(unix)] path.push("/");
+        #[cfg(windows)]
+        path.push(r"C:\");
+        #[cfg(unix)]
+        path.push("/");
         path.push("windows");
         path.push("..");
         path.push(".");
